"""A module for functions dealing with thresholding."""

from typing import List

from geneweaver.core.schema.score import GenesetScoreType


def check_threshold(geneset_score: GenesetScoreType, value: float) -> bool:
    """Check to see if a value falls within the score threshold.

    :param geneset_score: The geneset score type and threshold arguments.
    :param value: The value to check against.
    :return: A boolean that indicates if the value falls within the range specified by
            the threshold.
    """
    score_type = int(geneset_score.score_type)
    value = float(value)

    # P and Q values
    if score_type == 1 or score_type == 2:
        return value <= geneset_score.threshold

    # Correlation and effect scores
    elif score_type == 4 or score_type == 5:
        return geneset_score.threshold_low <= value <= geneset_score.threshold

    # Binary
    else:
        return value >= geneset_score.threshold


def check_threshold_list(
    geneset_score: GenesetScoreType, values: List[float]
) -> List[bool]:
    """Check to see if a list of values falls within the score threshold.

    :param geneset_score: The geneset score type and threshold arguments.
    :param values: The list of values to check against.
    :return: A list of booleans that indicates if the value falls within the range
            specified by the threshold.
    """
    return [check_threshold(geneset_score, value) for value in values]



"""Batch upload file rendering."""

from typing import List

from geneweaver.core.schema.batch import (
    HEADER_CHARACTERS,
    IGNORE_CHARACTERS,
    INV_CHAR_MAP,
    SPACE_SEPARATED_HEADER_CHARACTERS,
    BatchUploadGeneset,
)

CHAR_MAP = HEADER_CHARACTERS | SPACE_SEPARATED_HEADER_CHARACTERS | IGNORE_CHARACTERS


def format_batch_file(genesets: List[BatchUploadGeneset]) -> str:
    """Format a batch upload file from a list of genesets.

    :param genesets: A list of genesets to format.

    :return: A string containing the genesets in batch upload format.
    """
    data_str = ""
    for geneset in genesets:
        data_str += format_geneset(geneset)
        data_str += "\n"
    return data_str


def format_geneset_metadata(geneset: BatchUploadGeneset) -> str:
    """Format geneset metadata for a batch upload file.

    :param geneset: The geneset to format.

    :return: A string containing the geneset metadata in batch upload format.
    """
    data_str = "\n".join(
        (
            f"{INV_CHAR_MAP[key] if key in INV_CHAR_MAP else key} {str(value)}"
            for key, value in geneset
            if key != "values" and value is not None
        )
    )
    data_str += "\n\n"
    return data_str


def format_geneset_values(geneset: BatchUploadGeneset) -> str:
    """Format geneset values for a batch upload file.

    :param geneset: The geneset to format.

    :return: A string containing the geneset values in batch upload format.
    """
    return "\n".join((str(gene_value) for gene_value in geneset.values))  # noqa: PD011


def format_geneset(geneset: BatchUploadGeneset) -> str:
    """Format a geneset for a batch upload file.

    :param geneset: The geneset to format.

    :return: A string containing the geneset in batch upload format.
    """
    data_str = format_geneset_metadata(geneset)
    data_str += format_geneset_values(geneset)
    return data_str



"""Mapping definitions."""

from geneweaver.core.enum import GeneIdentifier, Species

AON_ID_TYPE_FOR_SPECIES = {
    Species.MUS_MUSCULUS: GeneIdentifier.MGI,
    Species.HOMO_SAPIENS: GeneIdentifier.HGNC,
    Species.RATTUS_NORVEGICUS: GeneIdentifier.RGD,
    Species.DANIO_RERIO: GeneIdentifier.ZFIN,
    Species.DROSOPHILA_MELANOGASTER: GeneIdentifier.FLYBASE,
    Species.MACACA_MULATTA: GeneIdentifier.ENTREZ,
    Species.CAENORHABDITIS_ELEGANS: GeneIdentifier.WORMBASE,
    Species.SACCHAROMYCES_CEREVISIAE: GeneIdentifier.SGD,
    Species.GALLUS_GALLUS: GeneIdentifier.CGNC,
    Species.CANIS_FAMILIARIS: GeneIdentifier.ENTREZ,
}
